﻿using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.MovementStrategy;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using System.Text;
using System.Threading.Tasks;

namespace ProjectAccordionBus.CollectionGenericObjects;

public abstract class AbstractCompany
{
    protected readonly int _placeSizeWidth = 240;

    protected readonly int _placeSizeHeight = 60;

    protected readonly int _pictureWidth;

    protected readonly int _pictureHeight;

    protected ICollectionGenericObjects<DrawningBus?> _collection = null;

    private int GetMaxCount => _pictureWidth / _placeSizeWidth * (_pictureHeight / _placeSizeHeight / 2 * 3);

    public AbstractCompany(int picWidth, int picHeigth, ICollectionGenericObjects<DrawningBus> collection)
    {
        _pictureWidth = picWidth;
        _pictureHeight = picHeigth;
        _collection = collection;
        _collection.MaxCount = GetMaxCount;
    }

    public static int operator +(AbstractCompany company, DrawningBus bus)
    {
        return company._collection.Insert(bus);
    }

    public static DrawningBus operator -(AbstractCompany company, int position)
    {
        return company._collection.Remove(position);
    }

    public DrawningBus? GetRandomObject()
    {
        Random rnd = new();
        return _collection?.Get(rnd.Next(GetMaxCount));
    }

    public Bitmap? Show()
    {
        Bitmap bitmap = new(_pictureWidth, _pictureHeight);
        Graphics graphics = Graphics.FromImage(bitmap);
        DrawBackground(graphics);


        for (int i = 0; i < (_collection?.Count ?? 0); i++)
        {
            DrawningBus? obj = _collection?.Get(i);
            SetObjectPosition(i, _collection?.Count ?? 0, obj);
            obj?.DrawTransport(graphics);
        }

        return bitmap;
    }

    protected abstract void DrawBackground(Graphics g);

    protected abstract void SetObjectPosition(int position, int MaxPos, DrawningBus? bus);
}
