﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ProjectAccordionBus.Entities;

namespace ProjectAccordionBus.Drawnings;

public static class ExtentionDrawningBus
{
    private static readonly string _separatorForObject = ":";

    /// <summary>
    /// Создание объекта из строки
    /// </summary>
    /// <param name="info">Строка с данными для создания объекта</param>
    /// <returns>Объект</returns>
    public static DrawningBus? CreateDrawningBus(this string info)
    {
        string[] strs = info.Split(_separatorForObject);
        EntityBus? bus = EntityAccordionBus.CreateEntityAccordionBus(strs);
        if (bus != null)
        {
            return new DrawningAccordionBus(bus);
        }

        bus = EntityBus.CreateEntityBus(strs);
        if (bus != null)
        {
            return new DrawningBus(bus);
        }

        return null;
    }

    /// <summary>
    /// Получение данных для сохранения в файл
    /// </summary>
    /// <param name="drawningBus">Сохраняемый объект</param>
    /// <returns>Строка с данными по объекту</returns>
    public static string GetDataForSave(this DrawningBus drawningBus)
    {
        string[]? array = drawningBus?.EntityBus?.GetStringRepresentation();

        if (array == null)
        {
            return string.Empty;
        }

        return string.Join(_separatorForObject, array);
    }
}
