﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Threading.Tasks;
using static System.Reflection.Metadata.BlobBuilder;

namespace ProjectAccordionBus.Entities;

public class EntityAccordionBus : EntityBus
{
    public Color AdditionalColor { get; private set; }

    public void SetAdditionalColor(Color color) => AdditionalColor = color;

    public bool Compartment { get; private set; }

    public bool Entrance { get; private set; }

    public bool Windows { get; private set; }

    /// <summary>
    /// Инициализация полей объекта-класса accordion bus
    /// </summary>
    /// <param name="additionalColor">Дополнительный цвет</param>
    /// <param name="compartment">Признак наличия доп. отсека с гармошкой</param>
    /// <param name="entrance">Признак наличия входа у доп. отсека</param>
    /// <param name="windows">Признак наличия окон у доп. отсека</param>

    public EntityAccordionBus(int Speed, double Weight, Color bodyColor, Color additionalColor, bool compartment, bool entrance, bool windows) : base(Speed, Weight, bodyColor)
    {
        AdditionalColor = additionalColor;
        Compartment = compartment; // отсек
        Entrance = entrance;
        Windows = windows;
    }

    public override string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityAccordionBus), Speed.ToString(), Weight.ToString(), BodyColor.Name,
            AdditionalColor.Name, Compartment.ToString(), Entrance.ToString(), Windows.ToString() };
    }

    public static EntityAccordionBus? CreateEntityAccordionBus(string[] strs)
    {
        if (strs.Length != 8 || strs[0] != nameof(EntityAccordionBus))
        {
            return null;
        }

        return new EntityAccordionBus(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]), Color.FromName(strs[4]),
            Convert.ToBoolean(strs[5]), Convert.ToBoolean(strs[6]), Convert.ToBoolean(strs[7]));
    }
}
