﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using static System.Windows.Forms.VisualStyles.VisualStyleElement;

namespace ProjectAccordionBus.Entities;
public class EntityBus
{
    public int Speed { get; private set; }

    public double Weight { get; private set; }

    public Color BodyColor { get; private set; }

    public void SetBodyColor(Color color) => BodyColor = color;

    public double Step => Speed * 50 / Weight;

    /// <summary>
    /// Конструктор сущности
    /// </summary>
    /// <param name="speed">Скорость</param>
    /// <param name="weight">Вес автомобиля</param>
    /// <param name="bodyColor">Основной цвет</param>

    public EntityBus(int speed, double weight, Color bodyColor)
    {
        Speed = speed;
        Weight = weight;
        BodyColor = bodyColor;
    }

    public virtual string[] GetStringRepresentation()
    {
        return new[] { nameof(EntityBus), Speed.ToString(), Weight.ToString(), BodyColor.Name};
    }

    /// <summary>
    /// Создание объекта из массива строк
    /// </summary>
    /// <param name="strs"></param>
    /// <returns></returns>
    public static EntityBus? CreateEntityBus(string[] strs)
    {
        if (strs.Length != 4 || strs[0] != nameof(EntityBus))
        {
            return null;
        }

        return new EntityBus(Convert.ToInt32(strs[1]), Convert.ToDouble(strs[2]), Color.FromName(strs[3]));
    }

}
