﻿using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.Entities;
using ProjectAccordionBus.MovementStrategy;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAccordionBus;
public partial class FormAccordionBus : Form
{

    private DrawningBus? _drawningBus;

    private AbstractStrategy? _strategy;

    public DrawningBus SetBus
    {
        set
        {
            _drawningBus = value;
            _drawningBus.SetPictureSize(pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    public FormAccordionBus()
    {
        InitializeComponent();
        _strategy = null;
    }

    private void Draw()
    {
        if (_drawningBus == null)
        {
            return;
        }

        Bitmap bmp = new(pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningBus.DrawTransport(gr);
        pictureBoxAccordionBus.Image = bmp;
    }

    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningBus.MoveTransport(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningBus.MoveTransport(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningBus.MoveTransport(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningBus.MoveTransport(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningBus == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableBus(_drawningBus), pictureBoxAccordionBus.Width, pictureBoxAccordionBus.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}



