﻿using ProjectAccordionBus.Drawnings;
using ProjectAccordionBus.Entities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ProjectAccordionBus;

public partial class FormBusConfig : Form
{
    /// <summary>
    /// Объект прорисовки класса
    /// </summary>
    private DrawningBus? _bus;

    private event BusDelegate? _busDelegate;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormBusConfig()
    {
        InitializeComponent();

        panelRed.MouseDown += panel_MouseDown;
        panelGreen.MouseDown += panel_MouseDown;
        panelBlue.MouseDown += panel_MouseDown;
        panelWhite.MouseDown += panel_MouseDown;
        panelBlack.MouseDown += panel_MouseDown;
        panelGray.MouseDown += panel_MouseDown;
        panelPurple.MouseDown += panel_MouseDown;
        panelYellow.MouseDown += panel_MouseDown;

        buttonCancel.Click += (sender, e) => Close();
    }

    public void AddEvent(BusDelegate busDelegate)
    {
        _busDelegate += busDelegate;
    }

    // <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _bus?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _bus?.SetPosition(15, 15);
        _bus?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void panelObjects_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действия при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void panelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _bus = new DrawningBus((int)numericUpDownSpeed.Value,
                (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _bus = new
                DrawningAccordionBus((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value,
                Color.White,
                Color.Black, checkBoxCompartment.Checked,
                checkBoxEntrance.Checked, checkBoxWindows.Checked);
                break;
        }
        labelBodyColor.BackColor = Color.Empty;
        labelAdditionalColor.BackColor = Color.Empty;
        DrawObject();
    }

    private void panel_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Control)?.DoDragDrop((sender as Control)?.BackColor ?? Color.Black, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void buttonAdd_Click(object sender, EventArgs e)
    {
        if (_bus != null)
        {
            _busDelegate?.Invoke(_bus);
            Close();
        }
    }

    private void labelBodyColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data.GetDataPresent(typeof(Color)))
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    private void labelBodyColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_bus != null)
        {
            _bus.EntityBus.SetBodyColor((Color)e.Data.GetData(typeof(Color)));
            DrawObject();
        }
    }

    private void labelAdditionalColor_DragEnter(object sender, DragEventArgs e)
    {
        if (_bus is DrawningBus)
        {
            if (e.Data.GetDataPresent(typeof(Color)))
            {
                e.Effect = DragDropEffects.Copy;
            }
            else
            {
                e.Effect = DragDropEffects.None;
            }
        }
    }

    private void labelAdditionalColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_bus?.EntityBus is EntityAccordionBus _accordionBus)
        {
            _accordionBus.SetAdditionalColor((Color)e.Data.GetData(typeof(Color)));
        }
        DrawObject();

    }

    private void labelBodyColor_Click(object sender, EventArgs e)
    {

    }
}
